@@ -186,6 +186,7 @@ setMethod("edgeData",
 setMethod("edgeData",
           signature(self="graphBAM", from="character", to="character", attr="character"),
           function(self, from, to, attr) {
+              #browser()
               edgeData.from <- edgeData(self, attr=attr, from=from)
               unrecognized.nodes <- setdiff(to, nodes(self))
               if(length(unrecognized.nodes) > 0) {
@@ -401,6 +402,7 @@ setReplaceMethod("edgeData",
                            from="character", to="character",
                            attr="character", value="ANY"),
                  function(self, from, to, attr, value) {
+                     #browser();
                      .verifyAttrName(attr, names(self@edgeData@defaults))
                      lenFrom <- length(from)
                      lenTo <- length(to)
@@ -495,19 +497,41 @@ setMethod("numNodes", signature("graphBAM"),
 
 setMethod("isAdjacent",
         signature(object="graphBAM", from="character", to="character"),
+
         function(object, from, to) {
+
+            if(length(from) > 1 & length(to) == 1)
+                to <- rep(to, length(from))
+
+            if(length(to) > 1 & length(from) == 1)
+                from <- rep(from, length(to))
+            
+            stopifnot(length(from) == length(to))
+            
             nodeNames <- object@nodes
-            req_ft <- .align_from_to(from, to, nodeNames)
-            if (!isDirected(object)) {
-                ## normalize edges so that edges have nodes in lexical order
-                tmp <- .undirectEdges(req_ft[ , "from"], req_ft[ , "to"])
-                req_ft[ , "from"] <- tmp[["from"]]
-                req_ft[ , "to"] <- tmp[["to"]]
-            }
-            from_i <- match(req_ft[ , "from"], nodeNames)
-            to_i <- match(req_ft[ , "to"], nodeNames)
-            .Call(graph_bitarray_getBitCell, object@edgeSet@bit_vector, from_i, to_i)
-        })
+            edge.matrix <- edgeMatrix(object)
+            edge.count <- ncol(edge.matrix)
+            reciprocal.edges <- edgemode(object) == "undirected"
+
+            result <- rep(FALSE, length(from))
+            pair.count <- length(from)
+            
+            for(pc in seq_len(pair.count)) {
+               for(ec in seq_len(edge.count)){
+                  from.node <- nodeNames[edge.matrix["from", ec]]
+                  to.node   <- nodeNames[edge.matrix["to", ec]]
+                  if(from.node == from[pc] & to.node == to[pc]){
+                      result[pc] <- TRUE
+                      break;
+                      }
+                  if(reciprocal.edges & to.node == from[pc] & from.node == to[pc]){
+                     result[pc] <- TRUE
+                     break;
+                     }
+                  } # for ec
+              } # for pc
+            result  
+        }) # isAdjacent
 
 setMethod("subGraph", signature(snodes="character", graph="graphBAM"),
         function(snodes, graph){ 
@@ -552,6 +576,26 @@ setMethod("edgeMatrix", "graphBAM",
             t(df)
         })
 
+setMethod("adjacencyMatrix", "graphBAM",
+        function(object) {
+            reciprocal.edges <- edgemode(object) == "undirected";
+            em <- edgeMatrix(object)
+            nodes <- nodes(object)
+            node.count <- length(nodes)
+            mtx.adj <- matrix(0, nrow=node.count, ncol=node.count,
+                              dimnames=list(nodes, nodes))
+            for(col in 1:ncol(em)){
+               from <- em["from", col]
+               to   <- em["to",   col]
+               from.node <- nodes[from]
+               to.node   <- nodes[to]
+               mtx.adj[from.node, to.node] <- 1;
+               if(reciprocal.edges)
+                   mtx.adj[to.node, from.node] <- 1;
+               }
+            mtx.adj
+        })
+
 setMethod("clearNode",
         signature(node="character", object="graphBAM"),
         function(node, object) {
@@ -674,6 +718,7 @@ setMethod("addEdge",
                  ord <- list(newLeftPos = integer(0), newRightPos = integer(0), 
                           origLeftPos = seq_len(nt), origRightPos = seq_len(nt))
               }
+              #browser();
               newAttr <- vector(nt, mode = "numeric")
               newAttr[ord$origLeftPos] <- graph@edgeSet@weights[ord$origRightPos]
               newAttr[ord$newLeftPos] <- weights[ord$newRightPos]
@@ -792,13 +837,28 @@ setReplaceMethod("edgemode", c("graphBAM", "character"),
             graph@edgeSet@edge_attrs[[i]] <- graph@edgeSet@edge_attrs[[i]][ord$origLeftPos]
         }
     }
+    obv0 <- graph@edgeSet@bit_vector
     graph@edgeSet@bit_vector <- setBitCell(graph@edgeSet@bit_vector,
                                            match(from, nn),
                                            match(to, nn),
                                            rep(0L, nrow(req_ft)))
+    nbv0 <- graph@edgeSet@bit_vector
     ord <- .Call(graph_bitarray_getEdgeAttrOrder,  graph@edgeSet@bit_vector , 
             as.integer(req_from), as.integer(req_to))
     graph@edgeSet@weights <- graph@edgeSet@weights[ord$origLeftPos]
+
+    if(edgemode(graph) == "undirected"){ # reverse from/to
+       obv1 <- graph@edgeSet@bit_vector
+       graph@edgeSet@bit_vector <- setBitCell(graph@edgeSet@bit_vector,
+                                              match(to, nn),
+                                              match(from, nn),
+                                              rep(0L, nrow(req_ft)))
+       nbv1 <- graph@edgeSet@bit_vector
+       ord <- .Call(graph_bitarray_getEdgeAttrOrder,  graph@edgeSet@bit_vector , 
+                    as.integer(req_from), as.integer(req_to))
+       graph@edgeSet@weights <- graph@edgeSet@weights[ord$origLeftPos]
+       } # if undirected
+
     graph
 }
 
